package rs.dmi.master.component;

/**
 * Base class for all classes that represent element of a
 * {@link AbstractComponentCollection}.
 * 
 * @author niktrk
 * @date 20.02.2014.
 * 
 */
public abstract class AbstractComponent {

	protected int index;

	public AbstractComponent(int index) {
		this.index = index;
	}

	/**
	 * Get index of {@link AbstractComponent}. Index is basically its unique id.
	 * Two components are declared equal if they have the same index.
	 */
	public int getIndex() {
		return index;
	}

	@Override
	public int hashCode() {
		return index;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		AbstractComponent other = (AbstractComponent) obj;
		if (index != other.index) {
			return false;
		}
		return true;
	}

}
