package rs.dmi.master.component;

import java.util.Iterator;
import java.util.Map;

/**
 * Base class for all collections i.e. sets. It is actually a bitset implemented
 * with one long so it can contains maximum of 64 elements.
 * 
 * @author niktrk
 * @date 20.02.2014.
 * 
 */
public abstract class AbstractComponentCollection<T extends AbstractComponent> implements Iterable<T> {

	private long bitset;

	public AbstractComponentCollection() {
		bitset = 0L;
	}

	public AbstractComponentCollection(AbstractComponentCollection<T> collection) {
		bitset = collection.getBitset();
	}

	public AbstractComponentCollection(long bitset) {
		this.bitset = bitset;
	}

	public AbstractComponentCollection(String bitset) {
		this(Long.valueOf(bitset));
	}

	public long getBitset() {
		return bitset;
	}

	public int getSize() {
		return Long.bitCount(bitset);
	}

	public void add(T component) {
		bitset |= (1L << component.getIndex());
	}

	public void remove(T component) {
		bitset &= (~(1L << component.getIndex()));
	}

	public boolean contains(T component) {
		return (bitset & (1L << component.getIndex())) != 0L;
	}

	public void addCollection(AbstractComponentCollection<T> collection) {
		bitset |= collection.getBitset();
	}

	public boolean isSubsetOf(AbstractComponentCollection<T> collection) {
		return (bitset & collection.getBitset()) == bitset;
	}

	public long subtract(AbstractComponentCollection<T> collection) {
		return bitset & (~collection.getBitset());
	}

	@Override
	public int hashCode() {
		return (int) (bitset ^ (bitset >>> 32));
	}

	@Override
	@SuppressWarnings("unchecked")
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		if (bitset != ((AbstractComponentCollection<T>) obj).bitset) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		StringBuilder builder = new StringBuilder();
		for (T component : this) {
			builder.append(component);
		}
		return builder.toString();
	}

	@Override
	public Iterator<T> iterator() {
		return new Iterator<T>() {

			private long bitset = AbstractComponentCollection.this.bitset;
			private long index = 0;

			@Override
			public boolean hasNext() {
				return bitset != 0;
			}

			@Override
			public T next() {
				index = bitset & -bitset;
				bitset ^= index;
				return getBitComponentMap().get(index);
			}

			@Override
			public void remove() {
				AbstractComponentCollection.this.remove(getBitComponentMap().get(index));
			}

		};
	}

	/**
	 * Get {@link Map} that maps 1 << i to {@link Component} with index i.
	 */
	protected abstract Map<Long, T> getBitComponentMap();

}
