package rs.dmi.master.component.edge;

import java.util.HashMap;
import java.util.Map;

import rs.dmi.master.component.AbstractComponent;
import rs.dmi.master.component.vertex.Vertex;

/**
 * Graph edge.
 * 
 * @author niktrk
 * @date 21.10.2012.
 * 
 */
public class Edge extends AbstractComponent {

	public static final Edge[][] EDGES;
	public static final Map<Long, Edge> EDGES_BITMAP;
	public static final Edge ZERO_ONE_EDGE;

	static {
		EDGES = new Edge[Vertex.MAX_VERTICES][Vertex.MAX_VERTICES];
		EDGES_BITMAP = new HashMap<Long, Edge>();
		// edges[k][k] will stay null
		for (int i = 1; i < Vertex.MAX_VERTICES; i++) {
			for (int j = 0; j < i; j++) {
				EDGES[i][j] = new Edge(i, j);
				EDGES[j][i] = EDGES[i][j];
				EDGES_BITMAP.put(1L << EDGES[i][j].getIndex(), EDGES[i][j]);
			}
		}
		ZERO_ONE_EDGE = Edge.EDGES[0][1];
	}

	private int firstVertex;// adjacency matrix row
	private int secondVertex;// adjacency matrix column

	private Edge(int firstVertex, int secondVertex) {
		super(calculateEdgeIndex(firstVertex, secondVertex));
		this.firstVertex = Math.max(firstVertex, secondVertex);
		this.secondVertex = Math.min(firstVertex, secondVertex);
	}

	private static int calculateEdgeIndex(int firstVertex, int secondVertex) {
		int row = Math.max(firstVertex, secondVertex);
		int column = Math.min(firstVertex, secondVertex);
		//
		// *
		// **
		// ***
		// **..
		// 4,2 = 2,4
		int edgesBeforeRow = ((row - 1) * row) / 2;
		return edgesBeforeRow + column;
	}

	public int getFirstVertex() {
		return firstVertex;
	}

	public int getSecondVertex() {
		return secondVertex;
	}

	public Edge getMappedEdge(int[] vertexMap) {
		return EDGES[vertexMap[firstVertex]][vertexMap[secondVertex]];
	}

	@Override
	public String toString() {
		return "[" + firstVertex + "-" + secondVertex + "]";
	}

}
