package rs.dmi.master.component.vertex;

import java.util.HashMap;
import java.util.Map;

import rs.dmi.master.component.AbstractComponent;

/**
 * Graph vertex.
 * 
 * @author niktrk
 * @date 26.10.2012.
 * 
 */
public class Vertex extends AbstractComponent implements Comparable<Vertex> {

	public static final int MAX_VERTICES = 11;

	public static final Map<Long, Vertex> VERTICES_BITMAP;

	static {
		VERTICES_BITMAP = new HashMap<Long, Vertex>();
		for (int i = 0; i < MAX_VERTICES; i++) {
			VERTICES_BITMAP.put(1L << i, new Vertex(i));
		}
	}

	private int makerDeg;
	private int breakerDeg;

	public Vertex(int index) {
		super(index);
		makerDeg = 0;
		breakerDeg = 0;
	}

	public Vertex(Vertex vertex) {
		super(vertex.getIndex());
		makerDeg = vertex.getMakerDeg();
		breakerDeg = vertex.getBreakerDeg();
	}

	public void reset() {
		makerDeg = 0;
		breakerDeg = 0;
	}

	public int getMakerDeg() {
		return makerDeg;
	}

	public int getBreakerDeg() {
		return breakerDeg;
	}

	public void incMakerDeg() {
		makerDeg++;
	}

	public void decMakerDeg() {
		makerDeg--;
	}

	public void incBreakerDeg() {
		breakerDeg++;
	}

	public void decBreakerDeg() {
		breakerDeg--;
	}

	public void addNext() {
		makerDeg *= 10;
		breakerDeg *= 10;
	}

	@Override
	public int compareTo(Vertex other) {
		if (makerDeg < other.getMakerDeg()) {
			return -1;
		} else if (makerDeg > other.getMakerDeg()) {
			return 1;
		} else {
			if (breakerDeg < other.getBreakerDeg()) {
				return -1;
			} else if (breakerDeg > other.getBreakerDeg()) {
				return 1;
			} else {
				return 0;
			}
		}
	}

	@Override
	public String toString() {
		return "[" + index + "]";
	}

}
