package rs.dmi.master.game;

/**
 * Class describing how many moves {@link Player#MAKER} and
 * {@link Player#BREAKER} play in one turn.
 * 
 * @author niktrk
 * @date 25.01.2014.
 */
public class GamePlay {

	public static final GamePlay UNBIASED_MAKER_STARTS = new GamePlay(Player.MAKER, 1, 1);
	public static final GamePlay UNBIASED_BREAKER_STARTS = new GamePlay(Player.BREAKER, 1, 1);

	private Player firstPlayer;
	private Player secondPlayer;
	private int numberOfFirstPlayerMoves;
	private int period;

	public GamePlay(Player firstPlayer, int numberOfMakersMoves, int numberOfBreakersMoves) {
		this.firstPlayer = firstPlayer;
		if (Player.MAKER.equals(firstPlayer)) {
			numberOfFirstPlayerMoves = numberOfMakersMoves;
			secondPlayer = Player.BREAKER;
		} else {
			numberOfFirstPlayerMoves = numberOfBreakersMoves;
			secondPlayer = Player.MAKER;
		}
		this.period = numberOfMakersMoves + numberOfBreakersMoves;
	}

	public Player getNextPlayer(int usedEdgesNumber) {
		if (usedEdgesNumber % period < numberOfFirstPlayerMoves) {
			return firstPlayer;
		} else {
			return secondPlayer;
		}
	}

	public Player getFirstPlayer() {
		return firstPlayer;
	}

	public Player getSecondPlayer() {
		return secondPlayer;
	}

	public static final GamePlay getUnbiasedGamePlayForPlayer(String player) {
		char firstChar = player.toLowerCase().charAt(0);
		if (firstChar == 'm') {
			return UNBIASED_MAKER_STARTS;
		} else if (firstChar == 'b') {
			return UNBIASED_BREAKER_STARTS;
		} else {
			throw new IllegalArgumentException("Illegal player name: " + player);
		}
	}

}
