package rs.dmi.master.game;

import java.io.IOException;

/**
 * Class for holding statistics about game.
 * 
 * @author niktrk
 * @date 01.02.2014.
 * 
 */
public class GameStatisticsLogger {

	private long playCalls;

	private long lookupTableSize;
	private long lookupTableSizePartial;

	private long startTime;
	private long endTime;

	public GameStatisticsLogger() {
		playCalls = 0L;
		lookupTableSize = 0L;
		lookupTableSizePartial = 0L;
		startTime = 0L;
		endTime = 0L;
	}

	public void onNewLookupTableEntry() {
		lookupTableSize++;
		lookupTableSizePartial++;
		if (lookupTableSizePartial == 1000000L) {
			lookupTableSizePartial = 0L;
			System.out.println("In lookup table: " + lookupTableSize);
		}
	}

	public void onPlay() {
		playCalls++;
	}

	public void onGameStart(String gameName, int numberOfVertices, int depth) {
		System.out.println("Game: " + gameName);
		System.out.println("Game graph size: " + numberOfVertices + "; Depth: " + depth);
		startTime = System.currentTimeMillis();
	}

	public void onGameEnd(Player winner) {
		endTime = System.currentTimeMillis();
		System.out.println("Time: " + getGameTime());
		System.out.println("========================");
		System.out.println("====     " + winner + "     ====");
		System.out.println("========================");
		System.out.println("Total moves: " + playCalls);
		System.out.println("In lookup table: " + lookupTableSize);
		try {
			System.out.println("Press any key to exit...");
			System.in.read();
		} catch (IOException e) {
			// skip it
		}
	}

	public long getGameTime() {
		if (startTime != 0L && endTime != 0L) {
			return endTime - startTime;
		}
		return -1L;
	}

	public long getCurrentGameTime() {
		if (startTime != 0L) {
			return System.currentTimeMillis() - startTime;
		}
		return -1L;
	}

	public void onPathsCalculated(int numberOfPaths) {
		System.out.println("Total number of winning paths: " + numberOfPaths);
	}

}
