package rs.dmi.master.game.hamiltonian;

import java.util.Comparator;
import java.util.List;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.component.vertex.Vertex;
import rs.dmi.master.game.CompleteGameGraph;
import rs.dmi.master.game.Player;

/**
 * {@link Comparator} implementation for sorting free edges in hamiltonian
 * games.
 * 
 * @author niktrk
 * @date 23.02.2014.
 * 
 */
public class HamiltonianFreeEdgesComparator implements Comparator<Edge> {

	private final Player player;
	private final List<Vertex> vertexList;

	public HamiltonianFreeEdgesComparator(Player player, CompleteGameGraph completeGameGraph) {
		this.player = player;
		this.vertexList = completeGameGraph.getVertexList();
	}

	@Override
	public int compare(Edge first, Edge second) {
		if (Player.MAKER.equals(player)) {
			return Integer.compare(getTotalMakerDegree(first), getTotalMakerDegree(second));
		} else {
			return -Integer.compare(getTotalBreakerDegree(first), getTotalBreakerDegree(second));
		}
	}

	private int getTotalMakerDegree(Edge edge) {
		return getFirstVertex(edge).getMakerDeg() + getSecondVertex(edge).getMakerDeg();
	}

	private int getTotalBreakerDegree(Edge edge) {
		return getFirstVertex(edge).getBreakerDeg() + getSecondVertex(edge).getBreakerDeg();
	}

	private Vertex getFirstVertex(Edge edge) {
		return vertexList.get(edge.getFirstVertex());
	}

	private Vertex getSecondVertex(Edge edge) {
		return vertexList.get(edge.getSecondVertex());
	}

}
