package rs.dmi.master.game.hamiltonian;

import java.util.ArrayList;
import java.util.List;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.component.edge.EdgeCollection;
import rs.dmi.master.pathcollection.PathCollection;

/**
 * Hamiltonian cycle calculator for complete graph.
 * 
 * @author niktrk
 * @date 20.10.2012.
 * 
 */
public class HamiltonianPathCalculator {

	public static final HamiltonianPathCalculator INSTANCE = new HamiltonianPathCalculator();

	private List<Integer> currentPath;

	private HamiltonianPathCalculator() {
		super();
		currentPath = new ArrayList<Integer>();
	}

	public void getPaths(int numberOfVertices, PathCollection<?> pathCollection) {
		getPaths(numberOfVertices, -1, -1, pathCollection);
	}

	public void getPaths(int numberOfVertices, int start, int end, PathCollection<?> pathCollection) {
		calculatePaths(pathCollection, numberOfVertices, start, end);
	}

	public void getCycles(int numberOfVertices, PathCollection<?> cycleCollection) {
		getCycles(numberOfVertices, null, cycleCollection);
	}

	public void getCycles(int numberOfVertices, Edge edge, PathCollection<?> cycleCollection) {
		calculateCycles(cycleCollection, numberOfVertices, edge);
	}

	private void calculatePaths(PathCollection<?> pathCollection, int numberOfVertices, int start, int end) {
		// start paths from 0..numberOfVertices
		for (int i = 0; i < numberOfVertices; i++) {
			if (start == -1 || i == start) {
				currentPath.clear();
				currentPath.add(i);
				findPaths(pathCollection, numberOfVertices, end);
			}
		}
	}

	private void calculateCycles(PathCollection<?> pathCollection, int numberOfVertices, Edge edge) {
		currentPath.clear();
		currentPath.add(0);
		findCycles(pathCollection, numberOfVertices, edge);
	}

	private void findCycles(PathCollection<?> pathCollection, int numberOfVertices, Edge edge) {
		if (currentPath.size() == numberOfVertices) {

			currentPath.add(0);

			EdgeCollection cycle = new EdgeCollection(currentPath);
			if (edge == null || cycle.contains(edge)) {
				pathCollection.addPath(cycle);
			}

			currentPath.remove(currentPath.size() - 1);

		} else {
			for (int i = 0; i < numberOfVertices; i++) {
				// can be optimized by removing contains() but that doesn't
				// introduce any significant improvement
				if (!currentPath.contains(i)) {
					currentPath.add(i);
					findCycles(pathCollection, numberOfVertices, edge);
					currentPath.remove(currentPath.size() - 1);
				}
			}
		}
	}

	private void findPaths(PathCollection<?> pathCollection, int numberOfVertices, int end) {
		if (currentPath.size() == numberOfVertices) {

			if (end == -1 || currentPath.get(numberOfVertices - 1) == end) {
				pathCollection.addPath(new EdgeCollection(currentPath));
			}

		} else {
			for (int i = 0; i < numberOfVertices; i++) {
				// can be optimized by removing contains() but that doesn't
				// introduce any significant improvement
				if (!currentPath.contains(i)) {
					currentPath.add(i);
					findPaths(pathCollection, numberOfVertices, end);
					currentPath.remove(currentPath.size() - 1);
				}
			}
		}
	}

}
