package rs.dmi.master.game.hamiltonian.bitsetbased;

import java.util.BitSet;
import java.util.Comparator;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.game.CompleteGameGraph;
import rs.dmi.master.game.GamePlay;
import rs.dmi.master.game.Player;
import rs.dmi.master.game.base.AbstractBitSetPathCollectionGame;
import rs.dmi.master.game.hamiltonian.HamiltonianFreeEdgesComparator;
import rs.dmi.master.game.hamiltonian.HamiltonianPathCalculator;
import rs.dmi.master.lookuptable.BoundedLookupTable;
import rs.dmi.master.lookuptable.LookupTable;
import rs.dmi.master.lookuptable.ZeroOneLookupTable;
import rs.dmi.master.pathcollection.BitSetPathCollection;
import rs.dmi.master.pathcollection.PathCollection;

/**
 * Fixed Hamiltonian path game based on {@link BitSetPathCollection}.
 * 
 * @author niktrk
 * @date 18.07.2016.
 * 
 */
public class FixedHamiltonianPathGameBitSetBased extends AbstractBitSetPathCollectionGame {

	public FixedHamiltonianPathGameBitSetBased(int size, GamePlay gamePlay) {
		super(new CompleteGameGraph(size, gamePlay, Edge.ZERO_ONE_EDGE), new ZeroOneLookupTable());
	}

	public FixedHamiltonianPathGameBitSetBased(int size, GamePlay gamePlay, int lookupCapacity, int lookupDepth) {
		super(new CompleteGameGraph(size, gamePlay, Edge.ZERO_ONE_EDGE), new BoundedLookupTable<LookupTable>(lookupCapacity, lookupDepth) {

			@Override
			protected LookupTable createLookupTable() {
				return new ZeroOneLookupTable();
			}

		});
	}

	@Override
	protected int getMinimumNumberOfEdgesForMakerWin(int numberOfVertices) {
		return numberOfVertices - 1;
	}

	@Override
	protected void calculatePaths(int numberOfVertices, PathCollection<BitSet> pathCollection) {
		HamiltonianPathCalculator.INSTANCE.getPaths(numberOfVertices, 0, 1, pathCollection);
	}

	@Override
	public Comparator<Edge> getFreeEdgesComparatorForPlayer(Player player, CompleteGameGraph gameGraph) {
		return new HamiltonianFreeEdgesComparator(player, gameGraph);
	}

	public static void main(String[] args) {
		int size = Integer.valueOf(args[0]);
		int capacity = Integer.valueOf(args[1]);
		int depth = Integer.valueOf(args[2]);
		GamePlay gamePlay = GamePlay.getUnbiasedGamePlayForPlayer(args[3]);

		FixedHamiltonianPathGameBitSetBased game = new FixedHamiltonianPathGameBitSetBased(size, gamePlay, capacity, depth);
		game.playTheGame();
	}

}
