package rs.dmi.master.game.hamiltonian.bitsetbased;

import java.util.BitSet;
import java.util.Comparator;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.game.CompleteGameGraph;
import rs.dmi.master.game.GamePlay;
import rs.dmi.master.game.Player;
import rs.dmi.master.game.base.AbstractBitSetPathCollectionGame;
import rs.dmi.master.game.hamiltonian.HamiltonianFreeEdgesComparator;
import rs.dmi.master.game.hamiltonian.HamiltonianPathCalculator;
import rs.dmi.master.lookuptable.BoundedLookupTable;
import rs.dmi.master.lookuptable.LookupTable;
import rs.dmi.master.lookuptable.TLongSetMapLookupTable;
import rs.dmi.master.pathcollection.BitSetPathCollection;
import rs.dmi.master.pathcollection.PathCollection;

/**
 * Hamiltonian cycle game based on {@link BitSetPathCollection}.
 * 
 * @author niktrk
 * @date 05.02.2014.
 * 
 */
public class HamiltonianCycleGameBitSetBased extends AbstractBitSetPathCollectionGame {

	public HamiltonianCycleGameBitSetBased(int size, GamePlay gamePlay, LookupTable lookup) {
		super(size, gamePlay, lookup);
	}

	public HamiltonianCycleGameBitSetBased(CompleteGameGraph gameGraph, LookupTable lookup) {
		super(gameGraph, lookup);
	}

	@Override
	protected int getMinimumNumberOfEdgesForMakerWin(int numberOfVertices) {
		return numberOfVertices;
	}

	@Override
	protected void calculatePaths(int numberOfVertices, PathCollection<BitSet> cycleCollection) {
		HamiltonianPathCalculator.INSTANCE.getCycles(numberOfVertices, cycleCollection);
	}

	@Override
	public Comparator<Edge> getFreeEdgesComparatorForPlayer(Player player, CompleteGameGraph gameGraph) {
		return new HamiltonianFreeEdgesComparator(player, gameGraph);
	}

	public static void main(String[] args) {
		int size = Integer.valueOf(args[0]);
		int capacity = Integer.valueOf(args[1]);
		int depth = Integer.valueOf(args[2]);
		GamePlay gamePlay = GamePlay.getUnbiasedGamePlayForPlayer(args[3]);

		HamiltonianCycleGameBitSetBased game = new HamiltonianCycleGameBitSetBased(size, gamePlay, new BoundedLookupTable<LookupTable>(capacity,
				depth) {

			@Override
			protected LookupTable createLookupTable() {
				return new TLongSetMapLookupTable();
			}

		});
		game.playTheGame();
	}

}
