package rs.dmi.master.isomorphism;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.component.edge.EdgeCollection;
import rs.dmi.master.game.CompleteGameGraph;

/**
 * Game graph state object.
 * 
 * @author niktrk
 * @date 25.11.2012.
 * 
 */
public class GameGraphState implements Comparable<GameGraphState> {

	// edge to which 0-1 edge is transformed by vertex map
	private Edge zeroOneEdgeImage;

	private EdgeCollection makersEdges;
	private EdgeCollection breakersEdges;

	public GameGraphState() {
		super();
		makersEdges = new EdgeCollection();
		breakersEdges = new EdgeCollection();
	}

	public GameGraphState(CompleteGameGraph completeGameGraph) {
		zeroOneEdgeImage = Edge.ZERO_ONE_EDGE;
		makersEdges = completeGameGraph.getMakersEdges();
		breakersEdges = completeGameGraph.getBreakersEdges();
	}

	public EdgeCollection getMakersEdges() {
		return makersEdges;
	}

	public void setMakersEdges(EdgeCollection makersEdges) {
		this.makersEdges = makersEdges;
	}

	public EdgeCollection getBreakersEdges() {
		return breakersEdges;
	}

	public void setBreakersEdges(EdgeCollection breakersEdges) {
		this.breakersEdges = breakersEdges;
	}

	public Edge getZeroOneEdgeImage() {
		return zeroOneEdgeImage;
	}

	public void setZeroOneEdgeImage(Edge zeroOneEdgeImage) {
		this.zeroOneEdgeImage = zeroOneEdgeImage;
	}

	public int getSize() {
		return makersEdges.getSize() + breakersEdges.getSize();
	}

	@Override
	public int compareTo(GameGraphState other) {
		if (makersEdges.getBitset() < other.getMakersEdges().getBitset()) {
			return -1;
		} else if (makersEdges.getBitset() > other.getMakersEdges().getBitset()) {
			return 1;
		} else {
			if (breakersEdges.getBitset() < other.getBreakersEdges().getBitset()) {
				return -1;
			} else if (breakersEdges.getBitset() > other.getBreakersEdges().getBitset()) {
				return 1;
			} else {
				return 0;
			}
		}
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((breakersEdges == null) ? 0 : breakersEdges.hashCode());
		result = prime * result + ((makersEdges == null) ? 0 : makersEdges.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		GameGraphState other = (GameGraphState) obj;
		if (breakersEdges == null) {
			if (other.breakersEdges != null) {
				return false;
			}
		} else if (!breakersEdges.equals(other.breakersEdges)) {
			return false;
		}
		if (makersEdges == null) {
			if (other.makersEdges != null) {
				return false;
			}
		} else if (!makersEdges.equals(other.makersEdges)) {
			return false;
		}
		return true;
	}

	@Override
	public String toString() {
		return "Maker: " + makersEdges + " " + "Breaker: " + breakersEdges + "\n";
	}

}
