package rs.dmi.master.isomorphism.canonicallabel;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.game.CompleteGameGraph;
import rs.dmi.master.isomorphism.GameGraphState;

/**
 * Base class for all implementations of canonical labeling algorithms.
 * 
 * @author niktrk
 * @date 22.02.2014.
 * 
 */
public abstract class AbstractCanonicalLabelingAlgorithm {

	public AbstractCanonicalLabelingAlgorithm() {
		// default constructor
	}

	/**
	 * Relabel {@link CompleteGameGraph} using given int[] which represents
	 * vertex map. Return {@link GameGraphState}.
	 */
	protected static GameGraphState relabelGameGraph(CompleteGameGraph gameGraph, int[] vertexMap) {
		GameGraphState relabelled = new GameGraphState();

		relabelled.setZeroOneEdgeImage(Edge.ZERO_ONE_EDGE.getMappedEdge(vertexMap));

		for (Edge edge : gameGraph.getMakersEdges()) {
			relabelled.getMakersEdges().add(edge.getMappedEdge(vertexMap));
		}

		for (Edge edge : gameGraph.getBreakersEdges()) {
			relabelled.getBreakersEdges().add(edge.getMappedEdge(vertexMap));
		}

		return relabelled;
	}

	/**
	 * Get name of the algorithm.
	 */
	public abstract String getName();

	/**
	 * Compute a canonical graph for isomorphism class of given
	 * {@link CompleteGameGraph}.
	 */
	public abstract GameGraphState getCanonicalGraph(CompleteGameGraph completeGameGraph);

}
