package rs.dmi.master.isomorphism.canonicallabel;

import java.util.BitSet;

import rs.dmi.master.game.CompleteGameGraph;
import rs.dmi.master.isomorphism.GameGraphState;
import rs.dmi.master.isomorphism.canonicallabel.partition.Cell;
import rs.dmi.master.isomorphism.canonicallabel.partition.Partition;

/**
 * Isomorphism related functionality base class.
 * 
 * @author niktrk
 * @date 20.11.2012.
 * 
 */
public class NautyImpl extends AbstractCanonicalLabelingAlgorithm {

	private int nodeNumber;

	private int[] map;

	private GameGraphState first;
	private GameGraphState smallest;
	private BitSet firstRefinePath;
	private BitSet smallestRefinePath;

	private BitSet automorphismPath;

	public NautyImpl() {
		super();
	}

	@Override
	public GameGraphState getCanonicalGraph(CompleteGameGraph completeGameGraph) {
		init(completeGameGraph.getNumberOfVertices());

		Partition partition = new Partition(completeGameGraph.getVertexList());
		partition.refine(completeGameGraph);
		calculateCanonicalGraph(partition, completeGameGraph, new BitSet());

		return smallest;
	}

	private void init(int size) {
		nodeNumber = 0;
		map = new int[size];
		first = null;
		smallest = null;
		automorphismPath = null;
		firstRefinePath = null;
		smallestRefinePath = null;
	}

	private void calculateCanonicalGraph(Partition partition, CompleteGameGraph gameGraph, BitSet refinePath) {
		if (partition.size() == gameGraph.getNumberOfVertices()) {

			partition.calculateMap(map);
			GameGraphState mapped = relabelGameGraph(gameGraph, map);

			automorphismPath = null;
			if (first == null) {

				first = mapped;
				firstRefinePath = (BitSet) refinePath.clone();
				smallest = mapped;
				smallestRefinePath = firstRefinePath;

			} else if (mapped.compareTo(smallest) < 0) {

				smallest = mapped;
				smallestRefinePath = (BitSet) refinePath.clone();

			} else if (first.equals(mapped)) {
				automorphismPath = firstRefinePath;

			} else if (smallest.equals(mapped)) {
				automorphismPath = smallestRefinePath;
			}

		} else {
			int internalNodeNumber = nodeNumber;
			nodeNumber++;
			refinePath.set(internalNodeNumber);

			Cell firstSmallest = partition.getFirstSmallestCell();
			for (byte i = 0; i < firstSmallest.getVertices().size(); i++) {

				Partition refined = partition.artificiallyPartitionize(firstSmallest, firstSmallest.getVertex(i));
				refined.refine(gameGraph);

				calculateCanonicalGraph(refined, gameGraph, refinePath);

				// if automorphRefinePath is not null, we are going back up
				if (automorphismPath != null) {
					if (automorphismPath.get(internalNodeNumber)) {
						automorphismPath = null;
						continue;
					} else {
						break;
					}
				}

			}
			refinePath.clear(internalNodeNumber);
		}
	}

	@Override
	public String getName() {
		return "JavaNauty";
	}

}
