package rs.dmi.master.isomorphism.canonicallabel.partition;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import rs.dmi.master.component.vertex.Vertex;

/**
 * Cell of a partition, used in graph isomorphism algorithm.
 * 
 * @author niktrk
 * @date 22.11.2012.
 * 
 */
public class Cell {

	private List<Vertex> vertices;

	public Cell() {
		super();
		vertices = new ArrayList<Vertex>();
	}

	public Cell(List<Vertex> vertices) {
		this();
		for (byte i = 0; i < vertices.size(); i++) {
			this.vertices.add(new Vertex(vertices.get(i)));
		}
	}

	public List<Vertex> getVertices() {
		return vertices;
	}

	public Vertex getVertex(int index) {
		return vertices.get(index);
	}

	public void addVertex(Vertex vertex) {
		vertices.add(vertex);
	}

	public int size() {
		return vertices.size();
	}

	public Cell[] artificiallyPartitionize(Vertex vertex) {
		Cell first = new Cell();
		Cell second = new Cell();

		for (byte i = 0; i < vertices.size(); i++) {
			Vertex current = vertices.get(i);
			if (current.getIndex() == vertex.getIndex()) {
				first.addVertex(current);
			} else {
				second.addVertex(current);
			}
		}

		return new Cell[] { first, second };
	}

	public List<Cell> partitionize() {
		List<Cell> partitions = new ArrayList<Cell>();
		if (vertices.size() == 1) {

			partitions.add(this);

		} else if (vertices.size() == 2) {
			if (vertices.get(0).compareTo(vertices.get(1)) == 0) {

				partitions.add(this);

			} else {
				Cell first = new Cell();
				Cell second = new Cell();

				if ((vertices.get(0).compareTo(vertices.get(1)) < 0)) {
					first.addVertex(vertices.get(0));
					second.addVertex(vertices.get(1));
				} else {
					first.addVertex(vertices.get(1));
					second.addVertex(vertices.get(0));
				}

				partitions.add(first);
				partitions.add(second);
			}
		} else {

			Vertex[] arr = vertices.toArray(new Vertex[vertices.size()]);
			Arrays.sort(arr);

			Cell current = new Cell();
			current.addVertex(arr[0]);
			partitions.add(current);

			for (int i = 1; i < arr.length; i++) {
				if (arr[i].compareTo(current.getVertex(0)) != 0) {
					current = new Cell();
					partitions.add(current);
				}
				current.addVertex(arr[i]);
			}

		}
		return partitions;
	}

	@Override
	public String toString() {
		return vertices.toString();
	}

}
