package rs.dmi.master.isomorphism.canonicallabel.partition;

import java.util.ArrayList;
import java.util.List;

import rs.dmi.master.component.vertex.Vertex;
import rs.dmi.master.game.CompleteGameGraph;

/**
 * Ordered partition implementation using {@link Cell}.
 * 
 * @author niktrk
 * @date 19.02.2012.
 * 
 */
public class Partition {

	private List<Cell> cells;

	public Partition() {
		this.cells = new ArrayList<Cell>();
	}

	public Partition(List<Vertex> vertices) {
		// we assume that all vertices contain degrees already
		this.cells = new Cell(vertices).partitionize();
	}

	public void addCells(List<Cell> cells) {
		this.cells.addAll(cells);
	}

	public void addCell(Cell cell) {
		cells.add(cell);
	}

	public List<Cell> getCells() {
		return cells;
	}

	public Cell getCell(int index) {
		return cells.get(index);
	}

	public int size() {
		return cells.size();
	}

	public void calculateMap(int[] map) {
		for (byte i = 0; i < size(); i++) {
			map[getCell(i).getVertex(0).getIndex()] = i;
		}
	}

	public Partition artificiallyPartitionize(Cell firstSmallestCell, Vertex extract) {
		Partition partition = new Partition();

		for (byte i = 0; i < cells.size(); i++) {
			Cell current = getCell(i);

			if (current == firstSmallestCell) {
				Cell[] twoCells = current.artificiallyPartitionize(extract);
				partition.addCell(twoCells[0]);
				partition.addCell(twoCells[1]);
			} else {
				partition.addCell(current);
			}

		}

		return partition;
	}

	public Cell getFirstSmallestCell() {
		int firstSmallestCellSize = Integer.MAX_VALUE;
		Cell firstSmallestCell = null;

		for (byte i = 0; i < cells.size(); i++) {
			Cell current = getCell(i);
			if (current.size() > 1 && current.size() < firstSmallestCellSize) {
				firstSmallestCellSize = current.size();
				firstSmallestCell = current;
			}
		}

		return firstSmallestCell;
	}

	public void refine(CompleteGameGraph gameGraph) {
		Partition local = new Partition();
		Partition refined = this;

		while (local.size() != refined.size()) {
			local = refined;
			refined = new Partition();

			for (byte i = 0; i < local.size(); i++) {
				Cell cell = local.getCell(i);

				if (cell.size() == 1) {
					refined.addCell(cell);
				} else {

					for (byte j = 0; j < cell.size(); j++) {
						calculateDegree(cell.getVertex(j), local, gameGraph);
					}

					refined.addCells(cell.partitionize());
				}
			}
		}

		cells = refined.getCells();
	}

	private void calculateDegree(Vertex firstCellVertex, Partition local, CompleteGameGraph gameGraph) {
		firstCellVertex.reset();

		for (byte i = 0; i < local.size(); i++) {
			Cell secondCell = local.getCell(i);

			for (byte j = 0; j < secondCell.size(); j++) {
				Vertex secondCellVertex = secondCell.getVertex(j);

				if (firstCellVertex.getIndex() != secondCellVertex.getIndex()) {

					if (gameGraph.getMakersEdges().contains(firstCellVertex, secondCellVertex)) {
						firstCellVertex.incMakerDeg();
					} else if (gameGraph.getBreakersEdges().contains(firstCellVertex, secondCellVertex)) {
						firstCellVertex.incBreakerDeg();
					}

				}

			}

			firstCellVertex.addNext();
		}
	}

}
