package rs.dmi.master.isomorphism.canonicallabel.partition;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.BitSet;
import java.util.List;

import rs.dmi.master.component.vertex.Vertex;
import rs.dmi.master.component.vertex.VertexCollection;

/**
 * Unordered partition implementation using {@link BitSet}.
 * 
 * @author niktrk
 * @date 15.02.2012.
 * 
 */
public class UnorderedPartition {

	private List<VertexCollection> cells;

	public UnorderedPartition(int size) {
		cells = new ArrayList<VertexCollection>(size);
		for (int i = 0; i < size; i++) {
			cells.add(new VertexCollection());
			cells.get(i).add(new Vertex(i));
		}
	}

	private void merge(int firstVertex, int secondVertex) {
		VertexCollection firstBitset = cells.get(firstVertex);
		VertexCollection secondBitset = cells.get(secondVertex);

		// it safe to compare references
		if (firstBitset != secondBitset) {

			// c(a) := c(a) OR c(b);
			firstBitset.addCollection(secondBitset);

			// c(x | x in c(b)) := c(a);
			for (Vertex vertex : secondBitset) {
				cells.set(vertex.getIndex(), firstBitset);
			}

		}
	}

	public List<Vertex> getMinCellRepresentatives(Cell cell) {
		// optimize simple cases
		if (cell.size() == 1) {
			return cell.getVertices();
		} else if (cell.size() == 2) {
			Vertex first = cell.getVertex(0);
			Vertex second = cell.getVertex(1);

			if (getOrbit(first) == getOrbit(second)) {
				return Arrays.asList(first);
			} else {
				return cell.getVertices();
			}
		}

		List<Vertex> result = new ArrayList<Vertex>();
		VertexCollection orbit = new VertexCollection();
		for (byte i = 0; i < cell.size(); i++) {
			Vertex current = cell.getVertex(i);

			if (!orbit.contains(current)) {
				orbit.addCollection(getOrbit(current));
				result.add(current);
			}

		}
		return result;
	}

	public void update(int[] automorphism) {
		for (int i = 0; i < automorphism.length; i++) {
			merge(i, automorphism[i]);
		}
	}

	public VertexCollection getOrbit(Vertex vertex) {
		return cells.get(vertex.getIndex());
	}

	@Override
	public String toString() {
		return cells.toString();
	}

	public static void main(String[] args) {
		UnorderedPartition uop = new UnorderedPartition(5);
		System.out.println(uop);
		uop.update(new int[] { 0, 3, 1, 2, 4 });
		System.out.println(uop);
		Cell cell = new Cell(Arrays.asList(new Vertex(0), new Vertex(1), new Vertex(2), new Vertex(3), new Vertex(4)));
		System.out.println(uop.getMinCellRepresentatives(cell));
		cell = new Cell(Arrays.asList(new Vertex(2), new Vertex(1)));
		System.out.println(uop.getMinCellRepresentatives(cell));
	}

}
