package rs.dmi.master.lookuptable;

import rs.dmi.master.game.Player;
import rs.dmi.master.isomorphism.GameGraphState;

/**
 * Abstract {@link LookupTable} with bound parameter, which is backed by any
 * other {@link LookupTable}.
 * 
 * @author niktrk
 * @date 02.01.2013.
 * 
 */
public abstract class BoundedLookupTable<T extends LookupTable> implements LookupTable {

	private final int maxEntries;
	private final int depth;

	private T firstTable;
	private T secondTable;

	private int firstEntries;
	private int secondEntries;

	public BoundedLookupTable(int maxEntries, int depth) {
		this.maxEntries = maxEntries;
		this.depth = depth;
		firstTable = createLookupTable();
		secondTable = createLookupTable();
		firstEntries = 0;
		secondEntries = 0;
	}

	@Override
	public Player getWinner(GameGraphState gameGraphState) {
		return getLookupTable(gameGraphState).getWinner(gameGraphState);
	}

	@Override
	public void setWinner(GameGraphState gameGraphState, Player winner) {
		newEntry(gameGraphState);
		if ((firstEntries + secondEntries) == maxEntries) {
			secondTable = createLookupTable();
			System.out.println("clean " + firstEntries + " " + secondEntries);
			secondEntries = 0;
		}
		getLookupTable(gameGraphState).setWinner(gameGraphState, winner);
	}

	private T getLookupTable(GameGraphState gameGraphState) {
		if (gameGraphState.getSize() <= depth) {
			return firstTable;
		} else {
			return secondTable;
		}
	}

	private void newEntry(GameGraphState gameGraphState) {
		if (gameGraphState.getSize() <= depth) {
			firstEntries++;
		} else {
			secondEntries++;
		}
	}

	/**
	 * Used to create new instance of concrete {@link LookupTable}.
	 */
	protected abstract T createLookupTable();

}
