package rs.dmi.master.lookuptable;

import gnu.trove.impl.Constants;
import gnu.trove.map.TLongByteMap;
import gnu.trove.map.TLongObjectMap;
import gnu.trove.map.hash.TLongByteHashMap;
import gnu.trove.map.hash.TLongObjectHashMap;
import rs.dmi.master.game.Player;
import rs.dmi.master.isomorphism.GameGraphState;

/**
 * Basic lookup using {@link TLongObjectHashMap}.
 * 
 * @author niktrk
 * @date 27.01.2014.
 * 
 */
public class TBasicLookupTable implements LookupTable {

	private TLongObjectMap<TLongByteMap> lookup;

	public TBasicLookupTable() {
		super();
		lookup = new TLongObjectHashMap<TLongByteMap>();
	}

	@Override
	public Player getWinner(GameGraphState gameGraphState) {
		TLongByteMap temp = lookup.get(gameGraphState.getMakersEdges().getBitset());
		if (temp != null) {

			byte value = temp.get(gameGraphState.getBreakersEdges().getBitset());
			if (value != -1) {
				return Player.VALUES[value];
			}

		}
		return null;
	}

	@Override
	public void setWinner(GameGraphState gameGraphState, Player winner) {
		TLongByteMap temp = lookup.get(gameGraphState.getMakersEdges().getBitset());
		if (temp == null) {

			temp = new TLongByteHashMap(Constants.DEFAULT_CAPACITY, Constants.DEFAULT_LOAD_FACTOR, -1L, (byte) -1);
			lookup.put(gameGraphState.getMakersEdges().getBitset(), temp);

		}
		temp.put(gameGraphState.getBreakersEdges().getBitset(), (byte) winner.ordinal());
	}

}
