package rs.dmi.master.lookuptable;

import gnu.trove.map.TLongObjectMap;
import gnu.trove.map.hash.TLongObjectHashMap;
import gnu.trove.set.TLongSet;
import gnu.trove.set.hash.TLongHashSet;

import java.util.EnumMap;

import rs.dmi.master.game.Player;
import rs.dmi.master.isomorphism.GameGraphState;

/**
 * {@link LookupTable} implementation using {@link TLongObjectHashMap} in
 * combination with {@link TLongHashSet} array.
 * 
 * @author niktrk
 * @date 17.04.2014.
 * 
 */
public class TLongSetMapLookupTable implements LookupTable {

	private EnumMap<Player, TLongObjectMap<TLongSet>> lookup;

	public TLongSetMapLookupTable() {
		lookup = new EnumMap<Player, TLongObjectMap<TLongSet>>(Player.class);
		lookup.put(Player.MAKER, new TLongObjectHashMap<TLongSet>());
		lookup.put(Player.BREAKER, new TLongObjectHashMap<TLongSet>());
	}

	@Override
	public Player getWinner(GameGraphState gameGraphState) {
		TLongSet temp = lookup.get(Player.MAKER).get(gameGraphState.getMakersEdges().getBitset());
		if (temp != null) {

			if (temp.contains(gameGraphState.getBreakersEdges().getBitset())) {
				return Player.MAKER;
			}

		}
		temp = lookup.get(Player.BREAKER).get(gameGraphState.getMakersEdges().getBitset());
		if (temp != null) {

			if (temp.contains(gameGraphState.getBreakersEdges().getBitset())) {
				return Player.BREAKER;
			}

		}
		return null;
	}

	@Override
	public void setWinner(GameGraphState gameGraphState, Player winner) {
		TLongSet temp = lookup.get(winner).get(gameGraphState.getMakersEdges().getBitset());
		if (temp == null) {

			temp = new TLongHashSet();
			lookup.get(winner).put(gameGraphState.getMakersEdges().getBitset(), temp);

		}
		temp.add(gameGraphState.getBreakersEdges().getBitset());
	}

}
