package rs.dmi.master.lookuptable;

import java.util.HashMap;
import java.util.Map;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.component.vertex.Vertex;
import rs.dmi.master.game.Player;
import rs.dmi.master.isomorphism.GameGraphState;

/**
 * {@link LookupTable} implementation which takes special care of 0-1 edge and
 * its image made by canonical labeling algorithm.
 * 
 * @author niktrk
 * @date 15.07.2016.
 * 
 */
public class ZeroOneLookupTable implements LookupTable {

	private Map<Edge, LookupTable> zeroOneEdgeImageLookupTable;

	public ZeroOneLookupTable() {
		zeroOneEdgeImageLookupTable = new HashMap<Edge, LookupTable>();

		for (int i = 1; i < Vertex.MAX_VERTICES; i++) {
			for (int j = 0; j < i; j++) {
				zeroOneEdgeImageLookupTable.put(Edge.EDGES[i][j], new TLongSetMapLookupTable());
			}
		}
	}

	@Override
	public Player getWinner(GameGraphState gameGraphState) {
		return zeroOneEdgeImageLookupTable.get(gameGraphState.getZeroOneEdgeImage()).getWinner(gameGraphState);
	}

	@Override
	public void setWinner(GameGraphState gameGraphState, Player winner) {
		zeroOneEdgeImageLookupTable.get(gameGraphState.getZeroOneEdgeImage()).setWinner(gameGraphState, winner);
	}

}
