package rs.dmi.master.pathcollection;

import java.util.Iterator;
import java.util.Set;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.component.edge.EdgeCollection;

/**
 * Collection of paths abstract implementation based on {@link Set}.
 * 
 * @author niktrk
 * @date 24.10.2012.
 * 
 */
public abstract class AbstractSetPathCollection implements PathCollection<Set<EdgeCollection>> {

	private final Set<EdgeCollection> pathSet;

	public AbstractSetPathCollection() {
		pathSet = createNewSet();
	}

	public AbstractSetPathCollection(Set<EdgeCollection> pathSet) {
		this.pathSet = pathSet;
	}

	@Override
	public Set<EdgeCollection> removeAllPathsWithEdge(Edge edge) {
		Set<EdgeCollection> removed = createNewSet();
		for (Iterator<EdgeCollection> iterator = pathSet.iterator(); iterator.hasNext();) {
			EdgeCollection edgeCollection = iterator.next();
			if (edgeCollection.contains(edge)) {
				iterator.remove();
				removed.add(edgeCollection);
			}
		}
		return removed;
	}

	@Override
	public boolean containsSubsetOf(EdgeCollection edgeCollection) {
		for (EdgeCollection path : pathSet) {
			if (path.isSubsetOf(edgeCollection)) {
				return true;
			}
		}
		return false;
	}

	@Override
	public boolean containsOrWillContainSubsetOf(EdgeCollection edgeCollection, EdgeCollection graphFreeEdges, Edge lastAddedEdge) {
		// no smarter solution for this at the moment here
		// TODO use same technique as in BitSetPathCollection
		return containsSubsetOf(edgeCollection);
	}

	@Override
	public boolean isEmpty() {
		return pathSet.isEmpty();
	}

	@Override
	public void addPathCollection(Set<EdgeCollection> pathCollection) {
		pathSet.addAll(pathCollection);
	}

	@Override
	public void addPath(EdgeCollection edgeCollection) {
		pathSet.add(edgeCollection);
	}

	@Override
	public int size() {
		return pathSet.size();
	}

	protected abstract Set<EdgeCollection> createNewSet();

}
