package rs.dmi.master.pathcollection;

import java.util.ArrayList;
import java.util.BitSet;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import rs.dmi.master.component.edge.Edge;
import rs.dmi.master.component.edge.EdgeCollection;

/**
 * Collection of paths. Implementation based on {@link BitSet}.
 * 
 * @author niktrk
 * @date 27.10.2012.
 * 
 */
public class BitSetPathCollection implements PathCollection<BitSet> {

	// i-th bit set if contains i-th cycle
	private BitSet pathSet;

	// get all cycles for edge
	private Map<Edge, BitSet> pathSetByEdge;

	private int counter;
	// get i-th cycle
	private List<EdgeCollection> pathByIndex;
	// used only to check if cycle is already found
	private Set<EdgeCollection> paths;

	public BitSetPathCollection() {
		super();
		counter = -1;
		pathSet = new BitSet();
		pathByIndex = new ArrayList<EdgeCollection>();
		pathSetByEdge = new HashMap<Edge, BitSet>();
		paths = new LinkedHashSet<EdgeCollection>();
	}

	@Override
	public void addPath(EdgeCollection edgeCollection) {
		if (paths.add(edgeCollection)) {
			counter++;
			pathSet.set(counter);
			pathByIndex.add(edgeCollection);
			for (Edge edge : edgeCollection) {
				BitSet bitset = pathSetByEdge.get(edge);
				if (bitset == null) {
					bitset = new BitSet();
					pathSetByEdge.put(edge, bitset);
				}
				bitset.set(counter);
			}

		}
	}

	@Override
	public boolean containsSubsetOf(EdgeCollection edgeCollection) {
		for (int i = pathSet.nextSetBit(0); i >= 0; i = pathSet.nextSetBit(i + 1)) {

			if (pathByIndex.get(i).isSubsetOf(edgeCollection)) {
				return true;
			}

		}
		return false;
	}

	@Override
	public boolean containsOrWillContainSubsetOf(EdgeCollection edgeCollection, EdgeCollection graphFreeEdges, Edge lastAddedEdge) {
		BitSet pathSetCopy = (BitSet) pathSet.clone();
		pathSetCopy.and(pathSetByEdge.get(lastAddedEdge));
		long firstFound = 0;

		for (int i = pathSetCopy.nextSetBit(0); i >= 0; i = pathSetCopy.nextSetBit(i + 1)) {

			long subtract = pathByIndex.get(i).subtract(edgeCollection);
			if (subtract == 0) {
				return true;
				// check if subtract has only one edge which is also free
			} else if ((subtract & (subtract - 1)) == 0 && (graphFreeEdges.getBitset() & subtract) == subtract) {

				if (firstFound == 0) {
					firstFound = subtract;
				} else if (firstFound != subtract) {
					return true;
				}

			}

		}
		return false;
	}

	@Override
	public void addPathCollection(BitSet pathCollection) {
		pathSet.or(pathCollection);
	}

	@Override
	public BitSet removeAllPathsWithEdge(Edge edge) {
		BitSet toRemove = pathSetByEdge.get(edge);

		BitSet removed = (BitSet) pathSet.clone();
		removed.and(toRemove);

		pathSet.andNot(toRemove);

		return removed;
	}

	@Override
	public boolean isEmpty() {
		return pathSet.isEmpty();
	}

	@Override
	public int size() {
		return pathSet.cardinality();
	}

}
