package rs.dmi.master.pathcollection;

import gnu.trove.set.hash.THashSet;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;

import rs.dmi.master.component.edge.EdgeCollection;

/**
 * Generic {@link AbstractSetPathCollection} implementation that creates new
 * {@link Set} instances using reflection.
 * 
 * @author niktrk
 * @date 28.01.2014.
 * 
 */
@SuppressWarnings({ "unchecked", "rawtypes" })
public class SetPathCollection extends AbstractSetPathCollection {

	private final Class<? extends Set> clazz;

	public SetPathCollection(Class<? extends Set> clazz) {
		super(createInstance(clazz));
		this.clazz = clazz;
	}

	public Class<? extends Set> getClazz() {
		return clazz;
	}

	@Override
	protected Set<EdgeCollection> createNewSet() {
		return createInstance(clazz);
	}

	private static Set<EdgeCollection> createInstance(Class<? extends Set> setClazz) {
		if (LinkedHashSet.class.equals(setClazz)) {
			return new LinkedHashSet<EdgeCollection>();
		} else if (HashSet.class.equals(setClazz)) {
			return new HashSet<EdgeCollection>();
		} else if (THashSet.class.equals(setClazz)) {
			return new THashSet<EdgeCollection>();
		} else {
			System.out.println(setClazz.getName() + " is not handled directly so instance is created using reflection.");
			return createInstanceUsingReflection(setClazz);
		}
	}

	private static Set<EdgeCollection> createInstanceUsingReflection(Class<? extends Set> setClazz) {
		try {
			return setClazz.newInstance();
		} catch (InstantiationException | IllegalAccessException e) {
			throw new IllegalStateException(e);
		}
	}

	public static void main(String[] args) {
		SetPathCollection t = new SetPathCollection(THashSet.class);
		Set<EdgeCollection> createNewSet = t.createNewSet();
		System.out.println(createNewSet.getClass());
	}

}
